create or replace  function call_center.cc_calls_rbac_users_from_group(_domain_id int8, _access smallint, _groups int[]) returns int[]
as
$$
select array_agg(distinct am.member_id::int)
from directory.wbt_class c
         inner join directory.wbt_default_acl a on a.object = c.id
         join directory.wbt_auth_member am on am.role_id = a.grantor
where c.name = 'calls'
  and c.dc = _domain_id
  and a.access&_access = _access and a.subject = any(_groups)
$$ language sql immutable ;

create or replace function call_center.cc_calls_rbac_users(_domain_id int8, _user_id int8) returns int[]
as
$$
with x as materialized (
    select a.user_id, a.id agent_id, a.supervisor, a.domain_id
    from directory.wbt_user u
             inner join call_center.cc_agent a on a.user_id = u.id
    where u.id = _user_id
      and u.dc = _domain_id
)
select array_agg(distinct a.user_id::int) users
from x
         left join lateral (
    select a.user_id, a.auditor_ids && array [x.user_id] aud
    from call_center.cc_agent a
    where a.domain_id = x.domain_id
      and (a.user_id = x.user_id or (a.supervisor_ids && array [x.agent_id] and a.supervisor) or
           a.auditor_ids && array [x.user_id])

    union
    distinct

    select a.user_id, a.auditor_ids && array [x.user_id] aud
    from call_center.cc_team t
             inner join call_center.cc_agent a on a.team_id = t.id
    where t.admin_ids && array [x.agent_id]
      and x.domain_id = t.domain_id
    ) a on true
$$ language sql immutable ;


create or replace  function call_center.cc_calls_rbac_queues(_domain_id int8, _user_id int8, _groups int[]) returns int[]
as
$$
with x as (select a.user_id, a.id agent_id, a.supervisor, a.domain_id
           from directory.wbt_user u
                    inner join call_center.cc_agent a on a.user_id = u.id and a.domain_id = u.dc
           where u.id = _user_id
             and u.dc = _domain_id)
select array_agg(distinct t.queue_id)
from (select qs.queue_id::int as queue_id
      from x
               left join lateral (
          select a.id, a.auditor_ids && array [x.user_id] aud
          from call_center.cc_agent a
          where (a.user_id = x.user_id or (a.supervisor_ids && array [x.agent_id] and a.supervisor))
          union
          distinct
          select a.id, a.auditor_ids && array [x.user_id] aud
          from call_center.cc_team t
                   inner join call_center.cc_agent a on a.team_id = t.id
          where t.admin_ids && array [x.agent_id]
          ) a on true
               inner join call_center.cc_skill_in_agent sa on sa.agent_id = a.id
               inner join call_center.cc_queue_skill qs
                          on qs.skill_id = sa.skill_id and sa.capacity between qs.min_capacity and qs.max_capacity
      where sa.enabled
        and qs.enabled
      union distinct
      select q.id
      from call_center.cc_queue q
      where q.domain_id = _domain_id
        and q.grantee_id = any (_groups)) t
$$ language sql immutable ;;


create index concurrently  if not exists cc_calls_history_grantee_id_index
    on call_center.cc_calls_history
        using btree(grantee_id asc nulls last );

create or replace  function call_center.cc_calls_rbac_users_from_group(_domain_id int8, _access smallint, _groups int[]) returns int[]
as
$$
select array_agg(distinct am.member_id::int)
from directory.wbt_class c
         inner join directory.wbt_default_acl a on a.object = c.id
         join directory.wbt_auth_member am on am.role_id = a.grantor
where c.name = 'calls'
  and c.dc = _domain_id
  and a.access&_access = _access and a.subject = any(_groups)
$$ language sql immutable ;

create or replace function call_center.cc_calls_rbac_users(_domain_id int8, _user_id int8) returns int[]
as
$$
with x as materialized (
    select a.user_id, a.id agent_id, a.supervisor, a.domain_id
    from directory.wbt_user u
             inner join call_center.cc_agent a on a.user_id = u.id
    where u.id = _user_id
      and u.dc = _domain_id
)
select array_agg(distinct a.user_id::int) users
from x
         left join lateral (
    select a.user_id, a.auditor_ids && array [x.user_id] aud
    from call_center.cc_agent a
    where a.domain_id = x.domain_id
      and (a.user_id = x.user_id or (a.supervisor_ids && array [x.agent_id] and a.supervisor) or
           a.auditor_ids && array [x.user_id])

    union
    distinct

    select a.user_id, a.auditor_ids && array [x.user_id] aud
    from call_center.cc_team t
             inner join call_center.cc_agent a on a.team_id = t.id
    where t.admin_ids && array [x.agent_id]
      and x.domain_id = t.domain_id
    ) a on true
$$ language sql immutable ;


create or replace  function call_center.cc_calls_rbac_queues(_domain_id int8, _user_id int8, _groups int[]) returns int[]
as
$$
with x as (select a.user_id, a.id agent_id, a.supervisor, a.domain_id
           from directory.wbt_user u
                    inner join call_center.cc_agent a on a.user_id = u.id and a.domain_id = u.dc
           where u.id = _user_id
             and u.dc = _domain_id)
select array_agg(distinct t.queue_id)
from (select qs.queue_id::int as queue_id
      from x
               left join lateral (
          select a.id, a.auditor_ids && array [x.user_id] aud
          from call_center.cc_agent a
          where (a.user_id = x.user_id or (a.supervisor_ids && array [x.agent_id] and a.supervisor))
          union
          distinct
          select a.id, a.auditor_ids && array [x.user_id] aud
          from call_center.cc_team t
                   inner join call_center.cc_agent a on a.team_id = t.id
          where t.admin_ids && array [x.agent_id]
          ) a on true
               inner join call_center.cc_skill_in_agent sa on sa.agent_id = a.id
               inner join call_center.cc_queue_skill qs
                          on qs.skill_id = sa.skill_id and sa.capacity between qs.min_capacity and qs.max_capacity
      where sa.enabled
        and qs.enabled
      union distinct
      select q.id
      from call_center.cc_queue q
      where q.domain_id = _domain_id
        and q.grantee_id = any (_groups)) t
$$ language sql immutable ;;


create index concurrently  if not exists cc_calls_history_grantee_id_index
    on call_center.cc_calls_history
        using btree(grantee_id asc nulls last );


create unique index if not exists cc_distribute_stats_uidx on call_center.cc_distribute_stats using btree(queue_id, bucket_id nulls last );
create unique index if not exists cc_inbound_stats_uidx on call_center.cc_inbound_stats using btree(queue_id, bucket_id nulls last );
drop index if exists call_center.cc_agent_today_pause_cause_agent_idx;
create unique index if not exists cc_agent_today_pause_cause_agent_uidx
    on call_center.cc_agent_today_pause_cause (id, today, cause);
create unique index if not exists cc_agent_today_stats_uidx
    on call_center.cc_agent_today_stats (agent_id);



create or replace function call_center.cc_distribute_inbound_call_to_queue(_node_name character varying, _queue_id bigint, _call_id character varying, variables_ jsonb, bucket_id_ integer, _priority integer DEFAULT 0, _sticky_agent_id integer DEFAULT NULL::integer) returns record
    language plpgsql
as
$$
declare
    _timezone_id             int4;
    _discard_abandoned_after int4;
    _weight                  int4;
    dnc_list_id_
                             int4;
    _domain_id               int8;
    _calendar_id             int4;
    _queue_updated_at        int8;
    _team_updated_at         int8;
    _team_id_                int;
    _list_comm_id            int8;
    _enabled                 bool;
    _q_type                  smallint;
    _sticky                  bool;
    _call                    record;
    _attempt                 record;
    _number                  varchar;
    _max_waiting_size        int;
    _grantee_id              int8;
BEGIN
    select c.timezone_id,
           (payload ->> 'discard_abandoned_after')::int discard_abandoned_after,
           c.domain_id,
           q.dnc_list_id,
           q.calendar_id,
           q.updated_at,
           ct.updated_at,
           q.team_id,
           q.enabled,
           q.type,
           q.sticky_agent,
           (payload ->> 'max_waiting_size')::int        max_size,
           q.grantee_id
    from call_center.cc_queue q
             inner join flow.calendar c on q.calendar_id = c.id
             left join call_center.cc_team ct on q.team_id = ct.id
    where q.id = _queue_id
    into _timezone_id, _discard_abandoned_after, _domain_id, dnc_list_id_, _calendar_id, _queue_updated_at,
        _team_updated_at, _team_id_, _enabled, _q_type, _sticky, _max_waiting_size, _grantee_id;

    if
        not _q_type = 1 then
        raise exception 'queue not inbound';
    end if;

    if
        not _enabled = true then
        raise exception 'queue disabled';
    end if;

    select *
    from call_center.cc_calls c
    where c.id = _call_id
--   for update
    into _call;

    if
            _call.domain_id != _domain_id then
        raise exception 'the queue on another domain';
    end if;

    if
            _call.id isnull or _call.direction isnull then
        raise exception 'not found call';
    ELSIF
                _call.direction <> 'outbound' or _call.user_id notnull then
        _number = _call.from_number;
    else
        _number = _call.destination;
    end if;

--   raise  exception '%', _number;


    if
        not exists(select accept
                   from flow.calendar_check_timing(_domain_id, _calendar_id, null)
                            as x (name varchar, excepted varchar, accept bool, expire bool)
                   where accept
                     and excepted is null
                     and not expire)
    then
        raise exception 'number % calendar not working [%]', _number, _calendar_id;
    end if;


    if
            _max_waiting_size > 0 then
        if (select count(*)
            from call_center.cc_member_attempt aa
            where aa.queue_id = _queue_id
              and aa.bridged_at isnull
              and aa.leaving_at isnull
              and (bucket_id_ isnull or aa.bucket_id = bucket_id_)) >= _max_waiting_size then
            raise exception using
                errcode = 'MAXWS',
                message = 'Queue maximum waiting size';
        end if;
    end if;

    if
        dnc_list_id_ notnull then
        select clc.id
        into _list_comm_id
        from call_center.cc_list_communications clc
        where (clc.list_id = dnc_list_id_
            and clc.number = _number)
        limit 1;
    end if;

    if
        _list_comm_id notnull then
        raise exception 'number % banned', _number;
    end if;

    if
            _discard_abandoned_after > 0 then
        select case
                   when log.result = 'abandoned' then
                           extract(epoch from now() - log.leaving_at)::int8 + coalesce(_priority, 0)
                   else coalesce(_priority, 0)
                   end
        from call_center.cc_member_attempt_history log
        where log.leaving_at >= (now() - (_discard_abandoned_after || ' sec')::interval)
          and log.queue_id = _queue_id
          and log.destination ->> 'destination' = _number
        order by log.leaving_at desc
        limit 1
        into _weight;
    end if;

    if
            _sticky_agent_id notnull and _sticky then
        if not exists(select 1
                      from call_center.cc_agent a
                      where a.id = _sticky_agent_id
                        and a.domain_id = _domain_id
                        and a.status = 'online'
                        and exists(select 1
                                   from call_center.cc_skill_in_agent sa
                                            inner join call_center.cc_queue_skill qs
                                                       on qs.skill_id = sa.skill_id and qs.queue_id = _queue_id
                                   where sa.agent_id = _sticky_agent_id
                                     and sa.enabled
                                     and sa.capacity between qs.min_capacity and qs.max_capacity)
            ) then
            _sticky_agent_id = null;
        end if;
    else
        _sticky_agent_id = null;
    end if;

    insert into call_center.cc_member_attempt (domain_id, state, queue_id, team_id, member_id, bucket_id, weight,
                                               member_call_id, destination, node_id, sticky_agent_id,
                                               list_communication_id,
                                               parent_id)
    values (_domain_id, 'waiting', _queue_id, _team_id_, null, bucket_id_, coalesce(_weight, _priority), _call_id,
            jsonb_build_object('destination', _number),
            _node_name, _sticky_agent_id, null, _call.attempt_id)
    returning * into _attempt;

    update call_center.cc_calls
    set queue_id   = _attempt.queue_id,
        team_id    = _team_id_,
        attempt_id = _attempt.id,
        payload    = case when jsonb_typeof(variables_::jsonb) = 'object' then variables_ else coalesce(payload, '{}') end, --coalesce(variables_, '{}'),
        grantee_id = _grantee_id
    where id = _call_id
    returning * into _call;

    if
            _call.id isnull or _call.direction isnull then
        raise exception 'not found call';
    end if;

    return row (
        _attempt.id::int8,
        _attempt.queue_id::int,
        _queue_updated_at::int8,
        _attempt.destination::jsonb,
        variables_::jsonb,
        _call.from_name::varchar,
        _team_updated_at::int8,
        _call.id::varchar,
        _call.state::varchar,
        _call.direction::varchar,
        _call.destination::varchar,
        call_center.cc_view_timestamp(_call.timestamp)::int8,
        _call.app_id::varchar,
        _number::varchar,
        case
            when (_call.direction <> 'outbound'
                and _call.to_name:: varchar <> ''
                and _call.to_name:: varchar notnull)
                then _call.from_name::varchar
            else _call.to_name::varchar end,
        call_center.cc_view_timestamp(_call.answered_at)::int8,
        call_center.cc_view_timestamp(_call.bridged_at)::int8,
        call_center.cc_view_timestamp(_call.created_at)::int8
        );

END;
$$;

create or replace function call_center.cc_distribute_inbound_call_to_agent(_node_name character varying, _call_id character varying, variables_ jsonb, _agent_id integer DEFAULT NULL::integer) returns record
    language plpgsql
as
$$
declare
    _domain_id int8;
    _team_updated_at int8;
    _agent_updated_at int8;
    _team_id_ int;

    _call record;
    _attempt record;

    _a_status varchar;
    _a_channel varchar;
    _number varchar;
    _busy_ext bool;
BEGIN

    select *
    from call_center.cc_calls c
    where c.id = _call_id
--   for update
    into _call;

    if _call.id isnull or _call.direction isnull then
        raise exception 'not found call';
    end if;

    if _call.id isnull or _call.direction isnull then
        raise exception 'not found call';
    ELSIF _call.direction <> 'outbound' then
        _number = _call.from_number;
    else
        _number = _call.destination;
    end if;

    select
        a.team_id,
        t.updated_at,
        a.status,
        cac.channel,
        a.domain_id,
        (a.updated_at - extract(epoch from u.updated_at))::int8,
        exists (select 1 from call_center.cc_calls c where c.user_id = a.user_id and c.queue_id isnull and c.hangup_at isnull ) busy_ext
    from call_center.cc_agent a
             inner join call_center.cc_team t on t.id = a.team_id
             inner join call_center.cc_agent_channel cac on a.id = cac.agent_id
             inner join directory.wbt_user u on u.id = a.user_id
    where a.id = _agent_id -- check attempt
      and length(coalesce(u.extension, '')) > 0
        for update
    into _team_id_,
        _team_updated_at,
        _a_status,
        _a_channel,
        _domain_id,
        _agent_updated_at,
        _busy_ext
    ;

    if _call.domain_id != _domain_id then
        raise exception 'the queue on another domain';
    end if;

    if _team_id_ isnull then
        raise exception 'not found agent';
    end if;

    if not _a_status = 'online' then
        raise exception 'agent not in online';
    end if;

    if not _a_channel isnull  then
        raise exception 'agent is busy';
    end if;

    if _busy_ext then
        raise exception 'agent has external call';
    end if;


    insert into call_center.cc_member_attempt (domain_id, state, team_id, member_call_id, destination, node_id, agent_id, parent_id)
    values (_domain_id, 'waiting', _team_id_, _call_id, jsonb_build_object('destination', _number),
            _node_name, _agent_id, _call.attempt_id)
    returning * into _attempt;

    update call_center.cc_calls
    set team_id = _team_id_,
        attempt_id = _attempt.id,
        payload    = case when jsonb_typeof(variables_::jsonb) = 'object' then variables_ else coalesce(payload, '{}') end
    where id = _call_id
    returning * into _call;

    if _call.id isnull or _call.direction isnull then
        raise exception 'not found call';
    end if;

    return row(
        _attempt.id::int8,
        _attempt.destination::jsonb,
        variables_::jsonb,
        _call.from_name::varchar,
        _team_id_::int,
        _team_updated_at::int8,
        _agent_updated_at::int8,

        _call.id::varchar,
        _call.state::varchar,
        _call.direction::varchar,
        _call.destination::varchar,
        call_center.cc_view_timestamp(_call.timestamp)::int8,
        _call.app_id::varchar,
        _number::varchar,
        case when (_call.direction <> 'outbound'
            and _call.to_name::varchar <> ''
            and _call.to_name::varchar notnull)
                 then _call.from_name::varchar
             else _call.to_name::varchar end,
        call_center.cc_view_timestamp(_call.answered_at)::int8,
        call_center.cc_view_timestamp(_call.bridged_at)::int8,
        call_center.cc_view_timestamp(_call.created_at)::int8
        );
END;
$$;


create or replace function call_center.cc_distribute_direct_member_to_queue(_node_name character varying, _member_id bigint, _communication_id integer, _agent_id bigint)
    returns TABLE(id bigint, member_id bigint, result character varying, queue_id integer, queue_updated_at bigint, queue_count integer, queue_active_count integer, queue_waiting_count integer, resource_id integer, resource_updated_at bigint, gateway_updated_at bigint, destination jsonb, variables jsonb, name character varying, member_call_id character varying, agent_id bigint, agent_updated_at bigint, team_updated_at bigint, seq integer)
    language plpgsql
as
$$
declare
    _weight      int4;
    _destination jsonb;
BEGIN

    return query with attempts as (
        insert into call_center.cc_member_attempt (state, queue_id, member_id, destination, communication_idx, node_id, agent_id, resource_id,
                                                   bucket_id, seq, team_id, domain_id)
            select 1,
                   m.queue_id,
                   m.id,
                   m.communications -> (_communication_id::int2),
                   (_communication_id::int2),
                   _node_name,
                   _agent_id,
                   r.resource_id,
                   m.bucket_id,
                   m.attempts + 1,
                   q.team_id,
                   q.domain_id
            from call_center.cc_member m
                     inner join call_center.cc_queue q on q.id = m.queue_id
                     inner join lateral (
                select (t::call_center.cc_sys_distribute_type).resource_id
                from call_center.cc_sys_queue_distribute_resources r,
                     unnest(r.types) t
                where r.queue_id = m.queue_id
                  and (t::call_center.cc_sys_distribute_type).type_id =
                      (m.communications -> (_communication_id::int2) -> 'type' -> 'id')::int4
                limit 1
                ) r on true
                     left join call_center.cc_outbound_resource cor on cor.id = r.resource_id
            where m.id = _member_id
              and m.communications -> (_communication_id::int2) notnull
              and not exists(select 1 from call_center.cc_member_attempt ma where ma.member_id = _member_id)
            returning call_center.cc_member_attempt.*
    )
                 select a.id,
                        a.member_id,
                        null::varchar          result,
                        a.queue_id,
                        cq.updated_at as       queue_updated_at,
                        0::integer             queue_count,
                        0::integer             queue_active_count,
                        0::integer             queue_waiting_count,
                        a.resource_id::integer resource_id,
                        r.updated_at::bigint   resource_updated_at,
                        null::bigint           gateway_updated_at,
                        a.destination          destination,
                        cm.variables,
                        cm.name,
                        null::varchar,
                        a.agent_id::bigint     agent_id,
                        ag.updated_at::bigint  agent_updated_at,
                        t.updated_at::bigint   team_updated_at,
                        a.seq::int seq
                 from attempts a
                          left join call_center.cc_member cm on a.member_id = cm.id
                          inner join call_center.cc_queue cq on a.queue_id = cq.id
                          left join call_center.cc_outbound_resource r on r.id = a.resource_id
                          left join call_center.cc_agent ag on ag.id = a.agent_id
                          inner join call_center.cc_team t on t.id = ag.team_id;

    --raise notice '%', _attempt_id;

END;
$$;


--
-- Name: cc_distribute_direct_member_to_queue(character varying, bigint, integer, bigint); Type: FUNCTION; Schema: call_center; Owner: -
--

CREATE or replace FUNCTION call_center.cc_distribute_direct_member_to_queue(_node_name character varying, _member_id bigint, _communication_id integer, _agent_id bigint) RETURNS TABLE(id bigint, member_id bigint, result character varying, queue_id integer, queue_updated_at bigint, queue_count integer, queue_active_count integer, queue_waiting_count integer, resource_id integer, resource_updated_at bigint, gateway_updated_at bigint, destination jsonb, variables jsonb, name character varying, member_call_id character varying, agent_id bigint, agent_updated_at bigint, team_updated_at bigint, seq integer)
    LANGUAGE plpgsql
AS $$
declare
    _weight      int4;
    _destination jsonb;
BEGIN

    return query with attempts as (
        insert into call_center.cc_member_attempt (state, queue_id, member_id, destination, node_id, agent_id, resource_id,
                                                   bucket_id, seq, team_id, domain_id)
            select 1,
                   m.queue_id,
                   m.id,
                   m.communications -> (_communication_id::int2),
                   _node_name,
                   _agent_id,
                   r.resource_id,
                   m.bucket_id,
                   m.attempts + 1,
                   q.team_id,
                   q.domain_id
            from call_center.cc_member m
                     inner join call_center.cc_queue q on q.id = m.queue_id
                     inner join lateral (
                select (t::call_center.cc_sys_distribute_type).resource_id
                from call_center.cc_sys_queue_distribute_resources r,
                     unnest(r.types) t
                where r.queue_id = m.queue_id
                  and (t::call_center.cc_sys_distribute_type).type_id =
                      (m.communications -> (_communication_id::int2) -> 'type' -> 'id')::int4
                limit 1
                ) r on true
                     left join call_center.cc_outbound_resource cor on cor.id = r.resource_id
            where m.id = _member_id
              and m.communications -> (_communication_id::int2) notnull
              and not exists(select 1 from call_center.cc_member_attempt ma where ma.member_id = _member_id)
            returning call_center.cc_member_attempt.*
    )
                 select a.id,
                        a.member_id,
                        null::varchar          result,
                        a.queue_id,
                        cq.updated_at as       queue_updated_at,
                        0::integer             queue_count,
                        0::integer             queue_active_count,
                        0::integer             queue_waiting_count,
                        a.resource_id::integer resource_id,
                        r.updated_at::bigint   resource_updated_at,
                        null::bigint           gateway_updated_at,
                        a.destination          destination,
                        cm.variables,
                        cm.name,
                        null::varchar,
                        a.agent_id::bigint     agent_id,
                        ag.updated_at::bigint  agent_updated_at,
                        t.updated_at::bigint   team_updated_at,
                        a.seq::int seq
                 from attempts a
                          left join call_center.cc_member cm on a.member_id = cm.id
                          inner join call_center.cc_queue cq on a.queue_id = cq.id
                          left join call_center.cc_outbound_resource r on r.id = a.resource_id
                          left join call_center.cc_agent ag on ag.id = a.agent_id
                          inner join call_center.cc_team t on t.id = ag.team_id;

    --raise notice '%', _attempt_id;

END;
$$;



--
-- Name: cc_set_active_members(character varying); Type: FUNCTION; Schema: call_center; Owner: -
--

CREATE or replace FUNCTION call_center.cc_set_active_members(node character varying) RETURNS TABLE(id bigint, member_id bigint, result character varying, queue_id integer, queue_updated_at bigint, queue_count integer, queue_active_count integer, queue_waiting_count integer, resource_id integer, resource_updated_at bigint, gateway_updated_at bigint, destination jsonb, variables jsonb, name character varying, member_call_id character varying, agent_id integer, agent_updated_at bigint, team_updated_at bigint, list_communication_id bigint, seq integer)
    LANGUAGE plpgsql
AS $$
BEGIN
    return query update call_center.cc_member_attempt a
        set state = case when c.queue_type = 4 then 'offering' else 'waiting' end
            ,node_id = node
            ,last_state_change = now()
            ,list_communication_id = lc.id
            ,seq = c.attempts + 1
            ,waiting_other_numbers = c.waiting_other_numbers
        from (
            select c.id,
                   cq.updated_at                                            as queue_updated_at,
                   r.updated_at                                             as resource_updated_at,
                   call_center.cc_view_timestamp(gw.updated_at)             as gateway_updated_at,
                   c.destination                                            as destination,
                   cm.variables                                             as variables,
                   cm.name                                                  as member_name,
                   c.state                                                  as state,
                   cqs.member_count                                         as queue_cnt,
                   0                                                        as queue_active_cnt,
                   cqs.member_waiting                                       as queue_waiting_cnt,
                   (ca.updated_at - extract(epoch from u.updated_at))::int8 as agent_updated_at,
                   tm.updated_at                                            as team_updated_at,
                   cq.dnc_list_id,
                   cm.attempts,
                   x.cnt                                                    as waiting_other_numbers,
                   cq.type                                                  as queue_type
            from call_center.cc_member_attempt c
                     inner join call_center.cc_member cm on c.member_id = cm.id
                     left join lateral (
                select count(*) cnt
                from jsonb_array_elements(cm.communications) WITH ORDINALITY AS x(c, n)
                where coalesce((x.c -> 'stop_at')::int8, 0) < 1
                  and x.n != (c.communication_idx + 1)
                ) x on c.member_id notnull
                     inner join call_center.cc_queue cq on cm.queue_id = cq.id
                     left join call_center.cc_team tm on tm.id = cq.team_id
                     left join call_center.cc_outbound_resource r on r.id = c.resource_id
                     left join directory.sip_gateway gw on gw.id = r.gateway_id
                     left join call_center.cc_agent ca on c.agent_id = ca.id
                     left join call_center.cc_queue_statistics cqs on cq.id = cqs.queue_id
                     left join directory.wbt_user u on u.id = ca.user_id
            where c.state = 'idle'
              and c.leaving_at isnull
            order by cq.priority desc, c.weight desc
                for update of c, cm, cq skip locked
        ) c
            left join call_center.cc_list_communications lc on lc.list_id = c.dnc_list_id and
                                                               lc.number = c.destination ->> 'destination'
        where a.id = c.id --and node = 'call_center-igor'
        returning
            a.id::bigint as id,
            a.member_id::bigint as member_id,
            a.result as result,
            a.queue_id::int as qeueue_id,
            c.queue_updated_at::bigint as queue_updated_at,
            c.queue_cnt::int,
            c.queue_active_cnt::int,
            c.queue_waiting_cnt::int,
            a.resource_id::int as resource_id,
            c.resource_updated_at::bigint as resource_updated_at,
            c.gateway_updated_at::bigint as gateway_updated_at,
            c.destination,
            c.variables ,
            c.member_name,
            a.member_call_id,
            a.agent_id,
            c.agent_updated_at,
            c.team_updated_at,
            a.list_communication_id,
            a.seq;
END;
$$;

--
-- Name: cc_member_queue_id_created_at; Type: INDEX; Schema: call_center; Owner: -
--

CREATE INDEX if not exists cc_member_queue_id_created_at ON call_center.cc_member USING btree (queue_id, created_at DESC);



-- 02
create or replace  function call_center.cc_calls_rbac_users_from_group(_domain_id int8, _access smallint, _groups int[]) returns int[]
as
$$
select array_agg(distinct am.member_id::int)
from directory.wbt_class c
         inner join directory.wbt_default_acl a on a.object = c.id
         join directory.wbt_auth_member am on am.role_id = a.grantor
where c.name = 'calls'
  and c.dc = _domain_id
  and a.access&_access = _access and a.subject = any(_groups)
$$ language sql immutable ;

create or replace function call_center.cc_calls_rbac_users(_domain_id int8, _user_id int8) returns int[]
as
$$
with x as materialized (
    select a.user_id, a.id agent_id, a.supervisor, a.domain_id
    from directory.wbt_user u
             inner join call_center.cc_agent a on a.user_id = u.id
    where u.id = _user_id
      and u.dc = _domain_id
)
select array_agg(distinct a.user_id::int) users
from x
         left join lateral (
    select a.user_id, a.auditor_ids && array [x.user_id] aud
    from call_center.cc_agent a
    where a.domain_id = x.domain_id
      and (a.user_id = x.user_id or (a.supervisor_ids && array [x.agent_id] and a.supervisor) or
           a.auditor_ids && array [x.user_id])

    union
    distinct

    select a.user_id, a.auditor_ids && array [x.user_id] aud
    from call_center.cc_team t
             inner join call_center.cc_agent a on a.team_id = t.id
    where t.admin_ids && array [x.agent_id]
      and x.domain_id = t.domain_id
    ) a on true
$$ language sql immutable ;


create or replace  function call_center.cc_calls_rbac_queues(_domain_id int8, _user_id int8, _groups int[]) returns int[]
as
$$
with x as (select a.user_id, a.id agent_id, a.supervisor, a.domain_id
           from directory.wbt_user u
                    inner join call_center.cc_agent a on a.user_id = u.id and a.domain_id = u.dc
           where u.id = _user_id
             and u.dc = _domain_id)
select array_agg(distinct t.queue_id)
from (select qs.queue_id::int as queue_id
      from x
               left join lateral (
          select a.id, a.auditor_ids && array [x.user_id] aud
          from call_center.cc_agent a
          where (a.user_id = x.user_id or (a.supervisor_ids && array [x.agent_id] and a.supervisor))
          union
          distinct
          select a.id, a.auditor_ids && array [x.user_id] aud
          from call_center.cc_team t
                   inner join call_center.cc_agent a on a.team_id = t.id
          where t.admin_ids && array [x.agent_id]
          ) a on true
               inner join call_center.cc_skill_in_agent sa on sa.agent_id = a.id
               inner join call_center.cc_queue_skill qs
                          on qs.skill_id = sa.skill_id and sa.capacity between qs.min_capacity and qs.max_capacity
      where sa.enabled
        and qs.enabled
      union distinct
      select q.id
      from call_center.cc_queue q
      where q.domain_id = _domain_id
        and q.grantee_id = any (_groups)) t
$$ language sql immutable ;;


create index concurrently  if not exists cc_calls_history_grantee_id_index
    on call_center.cc_calls_history
        using btree(grantee_id asc nulls last );



-- TODO NEW

--
-- Name: appointment_widget(character varying); Type: FUNCTION; Schema: call_center; Owner: -
--

CREATE or replace FUNCTION call_center.appointment_widget(_uri character varying) RETURNS TABLE(profile jsonb, list jsonb)
    LANGUAGE sql ROWS 1
AS $$
with profile as (
    select
        (config['queue']->>'id')::int as queue_id,
        (config['communicationType']->>'id')::int as communication_type,
        (config->>'duration')::interval as duration,
        (config->>'days')::int as days,
        (config->>'availableAgents')::int as available_agents,
        string_to_array((b.metadata->>'allow_origin'), ',') as allow_origins,
        q.calendar_id,
        b.id,
        b.uri,
        b.dc as domain_id,
        c.timezone_id,
        tz.sys_name as timezone
    from chat.bot b
             inner join lateral (select (b.metadata->>'appointment')::jsonb as config) as cfx on true
             inner join call_center.cc_queue q on q.id = (config['queue']->>'id')::int
             inner join flow.calendar c on c.id = q.calendar_id
             inner join flow.calendar_timezones tz on tz.id = c.timezone_id
    where b.uri = _uri and b.enabled
    limit 1
), d as materialized (
    select  q.queue_id,
            q.duration,
            q.available_agents,
            q.timezone,
            x,
            (extract(isodow from x::timestamp)  ) - 1 as day,
            dy.*,
            min(dy.ss::time) over () mins,
            max(dy.se::time) over () maxe
    from profile  q ,
         flow.calendar_day_range(q.calendar_id, least(q.days, 7)) x
             left join lateral (
             select t.*, tz.sys_name, c.excepts
             from flow.calendar c
                      inner join flow.calendar_timezones tz on tz.id = c.timezone_id
                      inner join lateral unnest(c.accepts::flow.calendar_accept_time[]) t on true
             where c.id = q.calendar_id
               and not t.disabled
             order by 1 asc
             ) y on y.day = (extract(isodow from x)  ) - 1
             left join lateral (
             select (x + (y.start_time_of_day || 'm')::interval)::timestamp as ss,
                    case when date_bin(q.duration, (x + (y.end_time_of_day || 'm')::interval)::timestamp, x::timestamp) < (x + (y.end_time_of_day || 'm')::interval)::timestamp
                             then date_bin(q.duration, (x + (y.end_time_of_day || 'm')::interval)::timestamp, x::timestamp) - q.duration
                         else date_bin(q.duration, (x + (y.end_time_of_day || 'm')::interval)::timestamp, x::timestamp) - q.duration end as se
             ) dy on true
)
   , min_max as materialized (
    select
        queue_id,
        x,
        duration,
        min(ss)  min_ss,
        max(se)  max_se
    from d
    group by 1, 2, 3
)
   ,res as materialized (
    select
        mem.*
    from min_max
             left join lateral (
        select
            date_bin(min_max.duration, coalesce(ready_at, created_at), coalesce(ready_at, created_at)::date)::timestamp d,
            count(*) cnt
        from call_center.cc_member m
        where m.stop_at isnull
          and m.queue_id = min_max.queue_id
          and coalesce(ready_at, created_at) between min_max.min_ss and min_max.max_se
        group by 1
        ) mem on true
    where mem notnull
)
   , list as (
    select
        d.*,
        res.*,
        xx,
        case when xx < now() at time zone d.timezone or coalesce(res.cnt, 0) >= d.available_agents or xx < d.ss or xx > d.se then true
             else false end as reserved
    from d
             left join generate_series((d.x || ' ' || d.mins)::timestamp, (d.x || ' ' || d.maxe)::timestamp, d.duration) xx on true
             left join res on res.d = xx
    limit 10080
)
   , ranges AS (
    select
        to_char(list.x::date,'YYYY-MM-DD')::text as date,
        jsonb_agg(jsonb_build_object('time', to_char(list.xx::time, 'HH24:MI'), 'reserved', list.reserved) order by list.x, list.xx) as times
    from list
    group by 1
)
select
    row_to_json(p) as profile,
    jsonb_agg(row_to_json(r)) as list
from profile p
         left join lateral (
    select *
    from ranges
    ) r on true
group by p
$$;





--
-- Name: cc_attempt_abandoned(bigint, integer, integer, jsonb, boolean, boolean, boolean); Type: FUNCTION; Schema: call_center; Owner: -
--

CREATE or replace FUNCTION call_center.cc_attempt_abandoned(attempt_id_ bigint, _max_count integer DEFAULT 0, _next_after integer DEFAULT 0, vars_ jsonb DEFAULT NULL::jsonb, _per_number boolean DEFAULT false, exclude_dest boolean DEFAULT false, redial boolean DEFAULT false) RETURNS record
    LANGUAGE plpgsql
AS $$
declare
    attempt  call_center.cc_member_attempt%rowtype;
    member_stop_cause varchar;
begin
    update call_center.cc_member_attempt
    set leaving_at = now(),
        last_state_change = now(),
        result = case when offering_at isnull and resource_id notnull then 'failed' else 'abandoned' end,
        state = 'leaving'
    where id = attempt_id_
    returning * into attempt;

    if attempt.member_id notnull then
        update call_center.cc_member
        set last_hangup_at  = (extract(EPOCH from now() ) * 1000)::int8,
            last_agent      = coalesce(attempt.agent_id, last_agent),
            stop_at = case when (stop_cause notnull or
                                 case when _per_number is true then (attempt.waiting_other_numbers > 0 or (_max_count > 0 and coalesce((communications#>(format('{%s,attempts}', attempt.communication_idx::int)::text[]))::int, 0) + 1 < _max_count)) else (_max_count > 0 and (attempts + 1 < _max_count)) end
                )  then stop_at else  attempt.leaving_at end,
            stop_cause = case when (stop_cause notnull or
                                    case when _per_number is true then (attempt.waiting_other_numbers > 0 or (_max_count > 0 and coalesce((communications#>(format('{%s,attempts}', attempt.communication_idx::int)::text[]))::int, 0) + 1 < _max_count)) else (_max_count > 0 and (attempts + 1 < _max_count)) end
                ) then stop_cause else attempt.result end,
            ready_at = now() + (_next_after || ' sec')::interval,
            communications =  jsonb_set(communications, (array[attempt.communication_idx::int])::text[], communications->(attempt.communication_idx::int) ||
                                                                                                         jsonb_build_object('last_activity_at', (case when redial is true then 0 else extract(epoch  from attempt.leaving_at) * 1000 end )::int8::text::jsonb) ||
                                                                                                         jsonb_build_object('attempt_id', attempt_id_) ||
                                                                                                         jsonb_build_object('attempts', coalesce((communications#>(format('{%s,attempts}', attempt.communication_idx::int)::text[]))::int, 0) + 1) ||
                                                                                                         case when exclude_dest or (_per_number is true and coalesce((communications#>(format('{%s,attempts}', attempt.communication_idx::int)::text[]))::int, 0) + 1 >= _max_count) then jsonb_build_object('stop_at', (extract(EPOCH from now() ) * 1000)::int8) else '{}'::jsonb end
                ),
            variables = case when vars_ notnull then coalesce(variables::jsonb, '{}') || vars_ else variables end,
            attempts        = attempts + 1                     --TODO
        where id = attempt.member_id
        returning stop_cause into member_stop_cause;
    end if;


    return row(attempt.last_state_change::timestamptz, member_stop_cause::varchar, attempt.result::varchar);
end;
$$;





--
-- Name: cc_calls_rbac_queues(bigint, bigint, integer[]); Type: FUNCTION; Schema: call_center; Owner: -
--

CREATE or replace FUNCTION call_center.cc_calls_rbac_queues(_domain_id bigint, _user_id bigint, _groups integer[]) RETURNS integer[]
    LANGUAGE sql IMMUTABLE
AS $$
with x as (select a.user_id, a.id agent_id, a.supervisor, a.domain_id
           from directory.wbt_user u
                    inner join call_center.cc_agent a on a.user_id = u.id and a.domain_id = u.dc
           where u.id = _user_id
             and u.dc = _domain_id)
select array_agg(distinct t.queue_id)
from (select qs.queue_id::int as queue_id
      from x
               left join lateral (
          select a.id, a.auditor_ids && array [x.user_id] aud
          from call_center.cc_agent a
          where (a.user_id = x.user_id or (a.supervisor_ids && array [x.agent_id] and a.supervisor))
          union
          distinct
          select a.id, a.auditor_ids && array [x.user_id] aud
          from call_center.cc_team t
                   inner join call_center.cc_agent a on a.team_id = t.id
          where t.admin_ids && array [x.agent_id]
          ) a on true
               inner join call_center.cc_skill_in_agent sa on sa.agent_id = a.id
               inner join call_center.cc_queue_skill qs
                          on qs.skill_id = sa.skill_id and sa.capacity between qs.min_capacity and qs.max_capacity
      where sa.enabled
        and qs.enabled
      union distinct
      select q.id
      from call_center.cc_queue q
      where q.domain_id = _domain_id
        and q.grantee_id = any (_groups)) t
$$;


--
-- Name: cc_calls_rbac_users(bigint, bigint); Type: FUNCTION; Schema: call_center; Owner: -
--

CREATE or replace FUNCTION call_center.cc_calls_rbac_users(_domain_id bigint, _user_id bigint) RETURNS integer[]
    LANGUAGE sql IMMUTABLE
AS $$
with x as materialized (
    select a.user_id, a.id agent_id, a.supervisor, a.domain_id
    from directory.wbt_user u
             inner join call_center.cc_agent a on a.user_id = u.id
    where u.id = _user_id
      and u.dc = _domain_id
)
select array_agg(distinct a.user_id::int) users
from x
         left join lateral (
    select a.user_id, a.auditor_ids && array [x.user_id] aud
    from call_center.cc_agent a
    where a.domain_id = x.domain_id
      and (a.user_id = x.user_id or (a.supervisor_ids && array [x.agent_id] and a.supervisor) or
           a.auditor_ids && array [x.user_id])

    union
    distinct

    select a.user_id, a.auditor_ids && array [x.user_id] aud
    from call_center.cc_team t
             inner join call_center.cc_agent a on a.team_id = t.id
    where t.admin_ids && array [x.agent_id]
      and x.domain_id = t.domain_id
    ) a on true
$$;


--
-- Name: cc_calls_rbac_users_from_group(bigint, smallint, integer[]); Type: FUNCTION; Schema: call_center; Owner: -
--

CREATE or replace FUNCTION call_center.cc_calls_rbac_users_from_group(_domain_id bigint, _access smallint, _groups integer[]) RETURNS integer[]
    LANGUAGE sql IMMUTABLE
AS $$
select array_agg(distinct am.member_id::int)
from directory.wbt_class c
         inner join directory.wbt_default_acl a on a.object = c.id
         join directory.wbt_auth_member am on am.role_id = a.grantor
where c.name = 'calls'
  and c.dc = _domain_id
  and a.access&_access = _access and a.subject = any(_groups)
$$;


alter TABLE call_center.cc_email_profile add column params jsonb;
alter TABLE call_center.cc_email_profile add column auth_type character varying DEFAULT 'plain'::character varying NOT NULL;
alter TABLE call_center.cc_email_profile add column listen boolean DEFAULT false NOT NULL;

drop VIEW call_center.cc_email_profile_list;
--
-- Name: cc_email_profile_list; Type: VIEW; Schema: call_center; Owner: -
--
CREATE VIEW call_center.cc_email_profile_list AS
SELECT t.id,
       t.domain_id,
       call_center.cc_view_timestamp(t.created_at) AS created_at,
       call_center.cc_get_lookup(t.created_by, (cc.name)::character varying) AS created_by,
       call_center.cc_view_timestamp(t.updated_at) AS updated_at,
       call_center.cc_get_lookup(t.updated_by, (cu.name)::character varying) AS updated_by,
       call_center.cc_view_timestamp(t.last_activity_at) AS activity_at,
       t.name,
       t.imap_host,
       t.smtp_host,
       t.login,
       t.mailbox,
       t.smtp_port,
       t.imap_port,
       t.fetch_err AS fetch_error,
       t.fetch_interval,
       t.state,
       call_center.cc_get_lookup((t.flow_id)::bigint, s.name) AS schema,
       t.description,
       t.enabled,
       t.password,
       t.listen
FROM (((call_center.cc_email_profile t
    LEFT JOIN directory.wbt_user cc ON ((cc.id = t.created_by)))
    LEFT JOIN directory.wbt_user cu ON ((cu.id = t.updated_by)))
    LEFT JOIN flow.acr_routing_scheme s ON ((s.id = t.flow_id)));


--
-- Name: cc_calls_history_grantee_id_index; Type: INDEX; Schema: call_center; Owner: -
--

CREATE INDEX if not exists cc_calls_history_grantee_id_index ON call_center.cc_calls_history USING btree (grantee_id);


--
-- Name: cc_calls_history_queue_id_dom; Type: INDEX; Schema: call_center; Owner: -
--

CREATE INDEX if not exists cc_calls_history_queue_id_dom ON call_center.cc_calls_history USING btree (domain_id, created_at, user_id, queue_id);


ALTER TABLE ONLY call_center.cc_member
    drop CONSTRAINT if exists cc_member_cc_agent_id_fk;

ALTER TABLE ONLY call_center.cc_member
    ADD CONSTRAINT cc_member_cc_agent_id_fk FOREIGN KEY (agent_id) REFERENCES call_center.cc_agent(id) ON UPDATE SET NULL ON DELETE SET NULL;

ALTER TABLE chat.message ADD file_url text NULL;

alter table storage.upload_file_jobs add column view_name character varying;
alter table storage.files add column view_name character varying;

create or replace view call_center.cc_distribute_stage_1
            (id, type, strategy, team_id, buckets, types, resources, offset_ids, lim, domain_id, priority, sticky_agent,
             sticky_agent_sec, recall_calendar, wait_between_retries_desc, strict_circuit)
as
WITH queues AS MATERIALIZED (SELECT q_1.domain_id,
                                    q_1.id,
                                    q_1.calendar_id,
                                    q_1.type,
                                    q_1.sticky_agent,
                                    q_1.recall_calendar,
                                    CASE
                                        WHEN q_1.sticky_agent
                                            THEN COALESCE((q_1.payload -> 'sticky_agent_sec'::text)::integer, 30)
                                        ELSE NULL::integer
                                        END                                                                                  AS sticky_agent_sec,
                                    CASE
                                        WHEN q_1.strategy::text = 'lifo'::text THEN 1
                                        WHEN q_1.strategy::text = 'strict_fifo'::text THEN 2
                                        ELSE 0
                                        END                                                                                  AS strategy,
                                    q_1.priority,
                                    q_1.team_id,
                                    (q_1.payload -> 'max_calls'::text)::integer                                              AS lim,
                                    (q_1.payload -> 'wait_between_retries_desc'::text)::boolean                              AS wait_between_retries_desc,
                                    COALESCE((q_1.payload -> 'strict_circuit'::text)::boolean, false)                        AS strict_circuit,
                                    array_agg(
                                            ROW (m.bucket_id::integer, m.member_waiting::integer, m.op)::call_center.cc_sys_distribute_bucket
                                            ORDER BY cbiq.priority DESC NULLS LAST, cbiq.ratio DESC NULLS LAST, m.bucket_id) AS buckets,
                                    m.op
                             FROM (WITH mem AS MATERIALIZED (SELECT a.queue_id,
                                                                    a.bucket_id,
                                                                    count(*) AS member_waiting,
                                                                    false    AS op
                                                             FROM call_center.cc_member_attempt a
                                                             WHERE a.bridged_at IS NULL
                                                               AND a.leaving_at IS NULL
                                                               AND a.state::text = 'wait_agent'::text
                                                             GROUP BY a.queue_id, a.bucket_id
                                                             UNION ALL
                                                             SELECT q_2.queue_id,
                                                                    q_2.bucket_id,
                                                                    q_2.member_waiting,
                                                                    true AS op
                                                             FROM call_center.cc_queue_statistics q_2
                                                             WHERE q_2.member_waiting > 0)
                                   SELECT rank() OVER (PARTITION BY mem.queue_id ORDER BY mem.op) AS pos,
                                          mem.queue_id,
                                          mem.bucket_id,
                                          mem.member_waiting,
                                          mem.op
                                   FROM mem) m
                                      JOIN call_center.cc_queue q_1 ON q_1.id = m.queue_id
                                      LEFT JOIN call_center.cc_bucket_in_queue cbiq
                                                ON cbiq.queue_id = m.queue_id AND cbiq.bucket_id = m.bucket_id
                             WHERE m.member_waiting > 0
                               AND q_1.enabled
                               AND q_1.type > 0
                               AND m.pos = 1
                               AND (cbiq.bucket_id IS NULL OR NOT cbiq.disabled)
                             GROUP BY q_1.domain_id, q_1.id, q_1.calendar_id, q_1.type, m.op
                             LIMIT 1024),
     calend AS MATERIALIZED (SELECT c.id                                                                            AS calendar_id,
                                    queues.id                                                                       AS queue_id,
                                    CASE
                                        WHEN queues.recall_calendar AND
                                             NOT (tz.offset_id = ANY (array_agg(DISTINCT o1.id)))
                                            THEN array_agg(DISTINCT o1.id)::integer[] + tz.offset_id::integer
                                        ELSE array_agg(DISTINCT o1.id)::integer[]
                                        END                                                                         AS l,
                                    queues.recall_calendar AND
                                    NOT (tz.offset_id = ANY (array_agg(DISTINCT o1.id)))                            AS recall_calendar
                             FROM flow.calendar c
                                      LEFT JOIN flow.calendar_timezones tz ON tz.id = c.timezone_id
                                      JOIN queues ON queues.calendar_id = c.id
                                      JOIN LATERAL unnest(c.accepts) a(disabled, day, start_time_of_day, end_time_of_day)
                                           ON true
                                      JOIN flow.calendar_timezone_offsets o1 ON (a.day + 1) =
                                                                                date_part('isodow'::text, timezone(o1.names[1], now()))::integer AND
                                                                                (to_char(timezone(o1.names[1], now()), 'SSSS'::text)::integer /
                                                                                 60) >= a.start_time_of_day AND
                                                                                (to_char(timezone(o1.names[1], now()), 'SSSS'::text)::integer /
                                                                                 60) <= a.end_time_of_day
                             WHERE NOT a.disabled IS TRUE
                               and not exists(
                                     select 1
                                     from unnest(c.excepts) as x
                                     where not x.disabled is true
                                       and case
                                               when x.repeat is true then
                                                       to_char((current_timestamp AT TIME ZONE tz.sys_name)::date, 'MM-DD') =
                                                       to_char((to_timestamp(x.date / 1000) at time zone tz.sys_name)::date, 'MM-DD')
                                               else
                                                       (current_timestamp AT TIME ZONE tz.sys_name)::date =
                                                       (to_timestamp(x.date / 1000) at time zone tz.sys_name)::date
                                         end
                                 )
                             GROUP BY c.id, queues.id, queues.recall_calendar, tz.offset_id),
     resources AS MATERIALIZED (SELECT l_1.queue_id,
                                       array_agg(ROW (cor.communication_id, cor.id::bigint, (l_1.l & l2.x::integer[])::smallint[], cor.resource_group_id::integer)::call_center.cc_sys_distribute_type) AS types,
                                       array_agg(ROW (cor.id::bigint, (cor."limit" - used.cnt)::integer, cor.patterns)::call_center.cc_sys_distribute_resource)                                         AS resources,
                                       call_center.cc_array_merge_agg(l_1.l & l2.x::integer[])                                                                                                          AS offset_ids
                                FROM calend l_1
                                         JOIN (SELECT corg.queue_id,
                                                      corg.priority,
                                                      corg.resource_group_id,
                                                      corg.communication_id,
                                                      corg."time",
                                                      (corg.cor).id                      AS id,
                                                      (corg.cor)."limit"                 AS "limit",
                                                      (corg.cor).enabled                 AS enabled,
                                                      (corg.cor).updated_at              AS updated_at,
                                                      (corg.cor).rps                     AS rps,
                                                      (corg.cor).domain_id               AS domain_id,
                                                      (corg.cor).reserve                 AS reserve,
                                                      (corg.cor).variables               AS variables,
                                                      (corg.cor).number                  AS number,
                                                      (corg.cor).max_successively_errors AS max_successively_errors,
                                                      (corg.cor).name                    AS name,
                                                      (corg.cor).last_error_id           AS last_error_id,
                                                      (corg.cor).successively_errors     AS successively_errors,
                                                      (corg.cor).created_at              AS created_at,
                                                      (corg.cor).created_by              AS created_by,
                                                      (corg.cor).updated_by              AS updated_by,
                                                      (corg.cor).error_ids               AS error_ids,
                                                      (corg.cor).gateway_id              AS gateway_id,
                                                      (corg.cor).email_profile_id        AS email_profile_id,
                                                      (corg.cor).payload                 AS payload,
                                                      (corg.cor).description             AS description,
                                                      (corg.cor).patterns                AS patterns,
                                                      (corg.cor).failure_dial_delay      AS failure_dial_delay,
                                                      (corg.cor).last_error_at           AS last_error_at
                                               FROM calend calend_1
                                                        JOIN (SELECT DISTINCT cqr.queue_id,
                                                                              corig.priority,
                                                                              corg_1.id AS resource_group_id,
                                                                              corg_1.communication_id,
                                                                              corg_1."time",
                                                                              CASE
                                                                                  WHEN cor_1.enabled AND gw.enable
                                                                                      THEN ROW (cor_1.id, cor_1."limit", cor_1.enabled, cor_1.updated_at, cor_1.rps, cor_1.domain_id, cor_1.reserve, cor_1.variables, cor_1.number, cor_1.max_successively_errors, cor_1.name, cor_1.last_error_id, cor_1.successively_errors, cor_1.created_at, cor_1.created_by, cor_1.updated_by, cor_1.error_ids, cor_1.gateway_id, cor_1.email_profile_id, cor_1.payload, cor_1.description, cor_1.patterns, cor_1.failure_dial_delay, cor_1.last_error_at, NULL::jsonb)::call_center.cc_outbound_resource
                                                                                  WHEN cor2.enabled AND gw2.enable
                                                                                      THEN ROW (cor2.id, cor2."limit", cor2.enabled, cor2.updated_at, cor2.rps, cor2.domain_id, cor2.reserve, cor2.variables, cor2.number, cor2.max_successively_errors, cor2.name, cor2.last_error_id, cor2.successively_errors, cor2.created_at, cor2.created_by, cor2.updated_by, cor2.error_ids, cor2.gateway_id, cor2.email_profile_id, cor2.payload, cor2.description, cor2.patterns, cor2.failure_dial_delay, cor2.last_error_at, NULL::jsonb)::call_center.cc_outbound_resource
                                                                                  ELSE NULL::call_center.cc_outbound_resource
                                                                                  END   AS cor
                                                              FROM call_center.cc_queue_resource cqr
                                                                       JOIN call_center.cc_outbound_resource_group corg_1
                                                                            ON cqr.resource_group_id = corg_1.id
                                                                       JOIN call_center.cc_outbound_resource_in_group corig
                                                                            ON corg_1.id = corig.group_id
                                                                       JOIN call_center.cc_outbound_resource cor_1
                                                                            ON cor_1.id = corig.resource_id::integer
                                                                       JOIN directory.sip_gateway gw ON gw.id = cor_1.gateway_id
                                                                       LEFT JOIN call_center.cc_outbound_resource cor2
                                                                                 ON cor2.id = corig.reserve_resource_id AND cor2.enabled
                                                                       LEFT JOIN directory.sip_gateway gw2 ON gw2.id = cor2.gateway_id AND cor2.enabled
                                                              WHERE CASE
                                                                        WHEN cor_1.enabled AND gw.enable THEN cor_1.id
                                                                        WHEN cor2.enabled AND gw2.enable THEN cor2.id
                                                                        ELSE NULL::integer
                                                                        END IS NOT NULL
                                                              ORDER BY cqr.queue_id, corig.priority DESC) corg
                                                             ON corg.queue_id = calend_1.queue_id) cor
                                              ON cor.queue_id = l_1.queue_id
                                         JOIN LATERAL ( WITH times
                                                                 AS (SELECT (e.value -> 'start_time_of_day'::text)::integer AS start,
                                                                            (e.value -> 'end_time_of_day'::text)::integer   AS "end"
                                                                     FROM jsonb_array_elements(cor."time") e(value))
                                                        SELECT array_agg(DISTINCT t.id) AS x
                                                        FROM flow.calendar_timezone_offsets t,
                                                             times,
                                                             LATERAL ( SELECT timezone(t.names[1], CURRENT_TIMESTAMP) AS t) with_timezone
                                                        WHERE (to_char(with_timezone.t, 'SSSS'::text)::integer / 60) >=
                                                              times.start
                                                          AND (to_char(with_timezone.t, 'SSSS'::text)::integer / 60) <=
                                                              times."end") l2 ON l2.* IS NOT NULL
                                         LEFT JOIN LATERAL ( SELECT count(*) AS cnt
                                                             FROM (SELECT 1 AS cnt
                                                                   FROM call_center.cc_member_attempt c_1
                                                                   WHERE c_1.resource_id = cor.id
                                                                     AND (c_1.state::text <> ALL
                                                                          (ARRAY ['leaving'::character varying::text, 'processing'::character varying::text]))) c) used
                                                   ON true
                                WHERE cor.enabled
                                  AND (cor.last_error_at IS NULL OR cor.last_error_at <=
                                                                    (now() - ((cor.failure_dial_delay || ' s'::text)::interval)))
                                  AND (cor."limit" - used.cnt) > 0
                                GROUP BY l_1.queue_id)
SELECT q.id,
       q.type,
       q.strategy::smallint AS strategy,
       q.team_id,
       q.buckets,
       r.types,
       r.resources,
       CASE
           WHEN q.type = ANY ('{7,8}'::smallint[]) THEN calend.l
           ELSE r.offset_ids
           END              AS offset_ids,
       CASE
           WHEN q.lim = '-1'::integer THEN NULL::integer
           ELSE GREATEST((q.lim - COALESCE(l.usage, 0::bigint))::integer, 0)
           END              AS lim,
       q.domain_id,
       q.priority,
       q.sticky_agent,
       q.sticky_agent_sec,
       calend.recall_calendar,
       q.wait_between_retries_desc,
       q.strict_circuit
FROM queues q
         LEFT JOIN calend ON calend.queue_id = q.id
         LEFT JOIN resources r ON q.op AND r.queue_id = q.id
         LEFT JOIN LATERAL ( SELECT count(*) AS usage
                             FROM call_center.cc_member_attempt a
                             WHERE a.queue_id = q.id
                               AND a.state::text <> 'leaving'::text) l ON q.lim > 0
WHERE (q.type = ANY (ARRAY [1, 6, 7]))
   OR q.type = 8 AND GREATEST((q.lim - COALESCE(l.usage, 0::bigint))::integer, 0) > 0
   OR q.type = 5 AND NOT q.op
   OR q.op AND (q.type = ANY (ARRAY [2, 3, 4, 5])) AND r.* IS NOT NULL;


CREATE or replace FUNCTION call_center.cc_attempt_timeout(attempt_id_ bigint, hold_sec integer, result_ character varying, agent_status_ character varying, agent_hold_sec_ integer) RETURNS timestamp with time zone
    LANGUAGE plpgsql
AS $$
declare
    attempt call_center.cc_member_attempt%rowtype;
begin
    update call_center.cc_member_attempt
    set reporting_at = now(),
        result = 'timeout',
        state = 'leaving'
    where id = attempt_id_
    returning * into attempt;

    update call_center.cc_member
    set last_hangup_at  = extract(EPOCH from now())::int8 * 1000,
        last_agent      = coalesce(attempt.agent_id, last_agent),


        stop_at = case when stop_at notnull or (q._max_count > 0 and (attempts + 1 < q._max_count))
                           then stop_at else  attempt.leaving_at end,
        stop_cause = case when stop_cause notnull or (q._max_count > 0 and (attempts + 1 < q._max_count))
                              then stop_cause else attempt.result end,
        ready_at = now() + (coalesce(q._next_after, 0) || ' sec')::interval,

        communications = jsonb_set(
                jsonb_set(communications, array [attempt.communication_idx, 'attempt_id']::text[],
                          attempt_id_::text::jsonb, true)
            , array [attempt.communication_idx, 'last_activity_at']::text[],
                ( (extract(EPOCH  from now()) * 1000)::int8 )::text::jsonb
            ),
        attempts        = attempts + 1
    from (
             -- fixme
             select coalesce(cast((q.payload->>'max_attempts') as int), 0) as _max_count, coalesce(cast((q.payload->>'wait_between_retries') as int), 0) as _next_after
             from call_center.cc_queue q
             where q.id = attempt.queue_id
         ) q
    where id = attempt.member_id;

    if attempt.agent_id notnull then
        update call_center.cc_agent_channel c
        set state = agent_status_,
            joined_at = now(),
            channel = case when c.channel = any('{chat,task}') and (select count(1)
                                                                    from call_center.cc_member_attempt aa
                                                                    where aa.agent_id = attempt.agent_id and aa.id != attempt.id and aa.state != 'leaving') > 0
                               then c.channel else null end,
            timeout = case when agent_hold_sec_ > 0 then (now() + (agent_hold_sec_::varchar || ' sec')::interval) else null end
        where c.agent_id = attempt.agent_id;

    end if;

    return now();
end;
$$;

-- new AI
alter table call_center.cc_calls
    drop column if exists amd_ml_result;
alter table call_center.cc_calls
    drop column if exists amd_ml_logs;

alter table call_center.cc_calls_history
    drop column if exists amd_ml_result;
alter table call_center.cc_calls_history
    drop column if exists amd_ml_logs;

alter table call_center.cc_calls
    add amd_ai_result varchar;
alter table call_center.cc_calls
    add amd_ai_logs varchar[];


alter table call_center.cc_calls_history
    add amd_ai_result varchar;
alter table call_center.cc_calls_history
    add amd_ai_logs varchar[];

alter table call_center.cc_calls add column amd_ai_positive bool;
alter table call_center.cc_calls_history add column amd_ai_positive bool;

drop view call_center.cc_calls_history_list;
create view call_center.cc_calls_history_list
as
SELECT c.id,
       c.app_id,
       'call'::character varying                                                                           AS type,
       c.parent_id,
       c.transfer_from,
       CASE
           WHEN c.parent_id IS NOT NULL AND c.transfer_to IS NULL AND c.id::text <> lega.bridged_id::text
               THEN lega.bridged_id
           ELSE c.transfer_to
           END                                                                                             AS transfer_to,
       call_center.cc_get_lookup(u.id, COALESCE(u.name, u.username::text)::character varying)              AS "user",
       CASE
           WHEN cq.type = ANY (ARRAY [4, 5]) THEN cag.extension
           ELSE u.extension
           END                                                                                             AS extension,
       call_center.cc_get_lookup(gw.id, gw.name)                                                           AS gateway,
       c.direction,
       c.destination,
       json_build_object('type', COALESCE(c.from_type, ''::character varying), 'number',
                         COALESCE(c.from_number, ''::character varying), 'id',
                         COALESCE(c.from_id, ''::character varying), 'name',
                         COALESCE(c.from_name, ''::character varying))                                     AS "from",
       json_build_object('type', COALESCE(c.to_type, ''::character varying), 'number',
                         COALESCE(c.to_number, ''::character varying), 'id', COALESCE(c.to_id, ''::character varying),
                         'name', COALESCE(c.to_name, ''::character varying))                               AS "to",
       c.payload                                                                                           AS variables,
       c.created_at,
       c.answered_at,
       c.bridged_at,
       c.hangup_at,
       c.stored_at,
       COALESCE(c.hangup_by, ''::character varying)                                                        AS hangup_by,
       c.cause,
       date_part('epoch'::text, c.hangup_at - c.created_at)::bigint                                        AS duration,
       COALESCE(c.hold_sec, 0)                                                                             AS hold_sec,
       COALESCE(
               CASE
                   WHEN c.answered_at IS NOT NULL THEN date_part('epoch'::text, c.answered_at - c.created_at)::bigint
                   ELSE date_part('epoch'::text, c.hangup_at - c.created_at)::bigint
                   END, 0::bigint)                                                                         AS wait_sec,
       CASE
           WHEN c.answered_at IS NOT NULL THEN date_part('epoch'::text, c.hangup_at - c.answered_at)::bigint
           ELSE 0::bigint
           END                                                                                             AS bill_sec,
       c.sip_code,
       f.files,
       call_center.cc_get_lookup(cq.id::bigint, cq.name)                                                   AS queue,
       call_center.cc_get_lookup(cm.id::bigint, cm.name)                                                   AS member,
       call_center.cc_get_lookup(ct.id, ct.name)                                                           AS team,
       call_center.cc_get_lookup(aa.id::bigint, COALESCE(cag.username, cag.name::name)::character varying) AS agent,
       cma.joined_at,
       cma.leaving_at,
       cma.reporting_at,
       cma.bridged_at                                                                                      AS queue_bridged_at,
       CASE
           WHEN cma.bridged_at IS NOT NULL THEN date_part('epoch'::text, cma.bridged_at - cma.joined_at)::integer
           ELSE date_part('epoch'::text, cma.leaving_at - cma.joined_at)::integer
           END                                                                                             AS queue_wait_sec,
       date_part('epoch'::text, cma.leaving_at - cma.joined_at)::integer                                   AS queue_duration_sec,
       cma.result,
       CASE
           WHEN cma.reporting_at IS NOT NULL THEN date_part('epoch'::text, cma.reporting_at - cma.leaving_at)::integer
           ELSE 0
           END                                                                                             AS reporting_sec,
       c.agent_id,
       c.team_id,
       c.user_id,
       c.queue_id,
       c.member_id,
       c.attempt_id,
       c.domain_id,
       c.gateway_id,
       c.from_number,
       c.to_number,
       c.tags,
       cma.display,
       (EXISTS(SELECT 1
               FROM call_center.cc_calls_history hp
               WHERE c.parent_id IS NULL
                 AND hp.parent_id::text = c.id::text))                                                     AS has_children,
       COALESCE(regexp_replace(cma.description::text, '^[\r\n\t ]*|[\r\n\t ]*$'::text, ''::text, 'g'::text),
                ''::character varying::text)::character varying                                            AS agent_description,
       c.grantee_id,
       holds.res                                                                                           AS hold,
       c.gateway_ids,
       c.user_ids,
       c.agent_ids,
       c.queue_ids,
       c.team_ids,
       (SELECT json_agg(row_to_json(annotations.*)) AS json_agg
        FROM (SELECT a.id,
                     a.call_id,
                     a.created_at,
                     call_center.cc_get_lookup(cc.id,
                                               COALESCE(cc.name, cc.username::text)::character varying)        AS created_by,
                     a.updated_at,
                     call_center.cc_get_lookup(uc.id,
                                               COALESCE(uc.name, uc.username::text)::character varying)        AS updated_by,
                     a.note,
                     a.start_sec,
                     a.end_sec
              FROM call_center.cc_calls_annotation a
                       LEFT JOIN directory.wbt_user cc ON cc.id = a.created_by
                       LEFT JOIN directory.wbt_user uc ON uc.id = a.updated_by
              WHERE a.call_id::text = c.id::text
              ORDER BY a.created_at DESC) annotations)                                                     AS annotations,
       COALESCE(c.amd_result, c.amd_ai_result)                                                             AS amd_result,
       c.amd_duration,
       c.amd_ai_result,
       c.amd_ai_logs,
       c.amd_ai_positive,
       cq.type                                                                                             AS queue_type,
       CASE
           WHEN c.parent_id IS NOT NULL THEN ''::text
           WHEN c.cause::text = ANY (ARRAY ['USER_BUSY'::character varying::text, 'NO_ANSWER'::character varying::text])
               THEN 'not_answered'::text
           WHEN c.cause::text = 'ORIGINATOR_CANCEL'::text THEN 'cancelled'::text
           WHEN c.cause::text = 'NORMAL_CLEARING'::text THEN
               CASE
                   WHEN c.cause::text = 'NORMAL_CLEARING'::text AND
                        (c.direction::text = 'outbound'::text AND c.hangup_by::text = 'A'::text AND
                         c.user_id IS NOT NULL OR
                         c.direction::text = 'inbound'::text AND c.hangup_by::text = 'B'::text AND
                         c.bridged_at IS NOT NULL OR
                         c.direction::text = 'outbound'::text AND c.hangup_by::text = 'B'::text AND
                         (cq.type = ANY (ARRAY [4, 5])) AND c.bridged_at IS NOT NULL) THEN 'agent_dropped'::text
                   ELSE 'client_dropped'::text
                   END
           ELSE 'error'::text
           END                                                                                             AS hangup_disposition,
       c.blind_transfer,
       (SELECT jsonb_agg(json_build_object('id', j.id, 'created_at', call_center.cc_view_timestamp(j.created_at),
                                           'action', j.action, 'file_id', j.file_id, 'state', j.state, 'error', j.error,
                                           'updated_at', call_center.cc_view_timestamp(j.updated_at))) AS jsonb_agg
        FROM storage.file_jobs j
        WHERE j.file_id = ANY (f.file_ids))                                                                AS files_job,
       transcripts.data                                                                                    AS transcripts,
       c.talk_sec,
       call_center.cc_get_lookup(au.id, au.name::character varying)                                        AS grantee
FROM call_center.cc_calls_history c
         LEFT JOIN LATERAL ( SELECT array_agg(f_1.id)                                                       AS file_ids,
                                    json_agg(jsonb_build_object('id', f_1.id, 'name', f_1.name, 'size', f_1.size,
                                                                'mime_type', f_1.mime_type, 'start_at',
                                                                (c.params -> 'record_start'::text)::bigint, 'stop_at',
                                                                (c.params -> 'record_stop'::text)::bigint)) AS files
                             FROM (SELECT f1.id,
                                          f1.size,
                                          f1.mime_type,
                                          f1.name
                                   FROM storage.files f1
                                   WHERE f1.domain_id = c.domain_id
                                     AND NOT f1.removed IS TRUE
                                     AND f1.uuid::text = c.id::text
                                   UNION ALL
                                   SELECT f1.id,
                                          f1.size,
                                          f1.mime_type,
                                          f1.name
                                   FROM storage.files f1
                                   WHERE f1.domain_id = c.domain_id
                                     AND NOT f1.removed IS TRUE
                                     AND f1.uuid::text = c.parent_id::text) f_1) f
                   ON c.answered_at IS NOT NULL OR c.bridged_at IS NOT NULL
         LEFT JOIN LATERAL ( SELECT jsonb_agg(x.hi ORDER BY (x.hi -> 'start'::text)) AS res
                             FROM (SELECT jsonb_array_elements(chh.hold) AS hi
                                   FROM call_center.cc_calls_history chh
                                   WHERE chh.parent_id::text = c.id::text
                                     AND chh.hold IS NOT NULL
                                   UNION
                                   SELECT jsonb_array_elements(c.hold) AS jsonb_array_elements) x
                             WHERE x.hi IS NOT NULL) holds ON c.parent_id IS NULL
         LEFT JOIN call_center.cc_queue cq ON c.queue_id = cq.id
         LEFT JOIN call_center.cc_team ct ON c.team_id = ct.id
         LEFT JOIN call_center.cc_member cm ON c.member_id = cm.id
         LEFT JOIN call_center.cc_member_attempt_history cma ON cma.id = c.attempt_id
         LEFT JOIN call_center.cc_agent aa ON cma.agent_id = aa.id
         LEFT JOIN directory.wbt_user cag ON cag.id = aa.user_id
         LEFT JOIN directory.wbt_user u ON u.id = c.user_id
         LEFT JOIN directory.sip_gateway gw ON gw.id = c.gateway_id
         LEFT JOIN directory.wbt_auth au ON au.id = c.grantee_id
         LEFT JOIN call_center.cc_calls_history lega ON c.parent_id IS NOT NULL AND lega.id::text = c.parent_id::text
         LEFT JOIN LATERAL ( SELECT json_agg(json_build_object('id', tr.id, 'locale', tr.locale, 'file_id', tr.file_id,
                                                               'file',
                                                               call_center.cc_get_lookup(ff.id, ff.name))) AS data
                             FROM storage.file_transcript tr
                                      LEFT JOIN storage.files ff ON ff.id = tr.file_id
                             WHERE tr.uuid::text = c.id::character varying(50)::text
                             GROUP BY (tr.uuid::text)) transcripts ON true;

drop materialized view call_center.cc_distribute_stats;
create materialized view call_center.cc_distribute_stats as
SELECT s.queue_id,
       s.bucket_id,
       s.start_stat,
       s.stop_stat,
       s.call_attempts,
       s.avg_handle,
       s.med_handle,
       s.avg_member_answer,
       s.avg_member_answer_not_bridged,
       s.avg_member_answer_bridged,
       s.max_member_answer,
       s.connected_calls,
       s.bridged_calls,
       s.abandoned_calls,
       s.connection_rate,
       s.over_dial,
       s.abandoned_rate,
       s.hit_rate,
       s.agents,
       s.aggent_ids
FROM call_center.cc_queue q
         LEFT JOIN LATERAL ( SELECT CASE
                                        WHEN ((q.payload -> 'amd'::text) -> 'allow_not_sure'::text)::boolean IS TRUE
                                            THEN ARRAY ['HUMAN'::text, 'NOTSURE'::text]
                                        ELSE ARRAY ['HUMAN'::text]
                                        END AS arr) amd ON true
         JOIN LATERAL ( SELECT att.queue_id,
                               att.bucket_id,
                               min(att.joined_at)                                                                       AS start_stat,
                               max(att.joined_at)                                                                       AS stop_stat,
                               count(*)                                                                                 AS call_attempts,
                               COALESCE(avg(date_part('epoch'::text,
                                                      COALESCE(att.reporting_at, att.leaving_at) - att.offering_at))
                                        FILTER (WHERE att.bridged_at IS NOT NULL),
                                        0::double precision)                                                            AS avg_handle,
                               COALESCE(avg(DISTINCT round(date_part('epoch'::text,
                                                                     COALESCE(att.reporting_at, att.leaving_at) -
                                                                     att.offering_at))::real)
                                        FILTER (WHERE att.bridged_at IS NOT NULL),
                                        0::double precision)                                                            AS med_handle,
                               COALESCE(avg(date_part('epoch'::text, ch.answered_at - att.joined_at))
                                        FILTER (WHERE ch.answered_at IS NOT NULL),
                                        0::double precision)                                                            AS avg_member_answer,
                               COALESCE(avg(date_part('epoch'::text, ch.answered_at - att.joined_at))
                                        FILTER (WHERE ch.answered_at IS NOT NULL AND ch.bridged_at IS NULL),
                                        0::double precision)                                                            AS avg_member_answer_not_bridged,
                               COALESCE(avg(date_part('epoch'::text, ch.answered_at - att.joined_at))
                                        FILTER (WHERE ch.answered_at IS NOT NULL AND ch.bridged_at IS NOT NULL),
                                        0::double precision)                                                            AS avg_member_answer_bridged,
                               COALESCE(max(date_part('epoch'::text, ch.answered_at - att.joined_at))
                                        FILTER (WHERE ch.answered_at IS NOT NULL),
                                        0::double precision)                                                            AS max_member_answer,
                               count(*) FILTER (WHERE ch.answered_at IS NOT NULL AND
                                                      amd_res.human) AS connected_calls,
                               count(*) FILTER (WHERE att.bridged_at IS NOT NULL)                                       AS bridged_calls,
                               count(*) FILTER (WHERE ch.answered_at IS NOT NULL AND att.bridged_at IS NULL AND
                                                      amd_res.human) AS abandoned_calls,
                               count(*) FILTER (WHERE ch.answered_at IS NOT NULL AND
                                                      amd_res.human)::double precision /
                               count(*)::double precision                                                               AS connection_rate,
                               CASE
                                   WHEN (count(*) FILTER (WHERE ch.answered_at IS NOT NULL AND
                                                                amd_res.human)::double precision /
                                         count(*)::double precision) > 0::double precision THEN 1::double precision / (
                                                   count(*) FILTER (WHERE ch.answered_at IS NOT NULL AND
                                                                          amd_res.human)::double precision /
                                                   count(*)::double precision) - 1::double precision
                                   ELSE (count(*) / GREATEST(count(DISTINCT att.agent_id), 1::bigint) - 1)::double precision
                                   END                                                                                  AS over_dial,
                               COALESCE((count(*)
                                         FILTER (WHERE ch.answered_at IS NOT NULL AND att.bridged_at IS NULL AND
                                                       amd_res.human)::double precision -
                                         COALESCE((q.payload -> 'abandon_rate_adjustment'::text)::integer,
                                                  0)::double precision) / NULLIF(count(*) FILTER (WHERE
                                       ch.answered_at IS NOT NULL AND
                                       amd_res.human),
                                                                                 0)::double precision *
                                        100::double precision,
                                        0::double precision)                                                            AS abandoned_rate,
                               count(*) FILTER (WHERE ch.answered_at IS NOT NULL AND
                                                      amd_res.human)::double precision /
                               count(*)::double precision                                                               AS hit_rate,
                               count(DISTINCT att.agent_id)                                                             AS agents,
                               array_agg(DISTINCT att.agent_id)
                               FILTER (WHERE att.agent_id IS NOT NULL)                                                  AS aggent_ids
                        FROM call_center.cc_member_attempt_history att
                                 LEFT JOIN call_center.cc_calls_history ch
                                           ON ch.domain_id = att.domain_id AND ch.id::text = att.member_call_id::text
                                 left join lateral (select (ch.amd_result IS NULL and ch.amd_ai_positive isnull ) or (ch.amd_result::text = ANY (amd.arr)) or ch.amd_ai_positive is true as human ) amd_res on true
                        WHERE att.channel::text = 'call'::text
                          AND att.joined_at > (now() - ((COALESCE((q.payload -> 'statistic_time'::text)::integer, 60) ||
                                                         ' min'::text)::interval))
                          AND att.queue_id = q.id
                          AND att.domain_id = q.domain_id
                        GROUP BY att.queue_id, att.bucket_id) s ON s.queue_id IS NOT NULL
WHERE q.type = 5
  AND q.enabled;


create unique index cc_distribute_stats_uidx
    on call_center.cc_distribute_stats (queue_id, bucket_id);
refresh materialized view call_center.cc_distribute_stats;


CREATE or replace PROCEDURE call_center.cc_distribute(INOUT cnt integer)
    LANGUAGE plpgsql
AS $$
begin
    if NOT pg_try_advisory_xact_lock(132132117) then
        raise exception 'LOCK';
    end if;

    with dis as MATERIALIZED (
        select x.*, a.team_id
        from call_center.cc_sys_distribute() x (agent_id int, queue_id int, bucket_id int, ins bool, id int8, resource_id int,
                                                resource_group_id int, comm_idx int)
                 left join call_center.cc_agent a on a.id= x.agent_id
    )
       , ins as (
        insert into call_center.cc_member_attempt (channel, member_id, queue_id, resource_id, agent_id, bucket_id, destination,
                                                   communication_idx, member_call_id, team_id, resource_group_id, domain_id, import_id, sticky_agent_id)
            select case when q.type = 7 then 'task' else 'call' end, --todo
                   dis.id,
                   dis.queue_id,
                   dis.resource_id,
                   dis.agent_id,
                   dis.bucket_id,
                   x,
                   dis.comm_idx,
                   uuid_generate_v4(),
                   dis.team_id,
                   dis.resource_group_id,
                   q.domain_id,
                   m.import_id,
                   case when q.type = 5 and q.sticky_agent then dis.agent_id end
            from dis
                     inner join call_center.cc_queue q on q.id = dis.queue_id
                     inner join call_center.cc_member m on m.id = dis.id
                     inner join lateral jsonb_extract_path(m.communications, (dis.comm_idx)::text) x on true
            where dis.ins
    )
    update call_center.cc_member_attempt a
    set agent_id = t.agent_id,
        team_id = t.team_id
    from (
             select dis.id, dis.agent_id, dis.team_id
             from dis

             where not dis.ins is true
         ) t
    where t.id = a.id
      and a.agent_id isnull;

end;
$$;


refresh materialized view call_center.cc_inbound_stats;
refresh materialized view call_center.cc_distribute_stats;
refresh materialized view call_center.cc_agent_today_stats;
refresh materialized view call_center.cc_agent_today_pause_cause;

